<?php

/*********************************************************************

 PHP Reply Ver 1.00 (2010/12/23)

*********************************************************************/

/********* ライセンス ************************************************

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

 @Link http://www.php-labo.net/
 @Copyright(C) 2007-2010 PHP Labo
 @Author Knight <info at php-labo dot net>

*********************************************************************/

/********* 設定項目 *************************************************/

//POP3サーバーのアドレス
define('POP_SERVER', 'pop3.example.com');

//POP3サーバーのユーザー名
define('POP_USER', 'user');

//POP3サーバーのパスワード
define('POP_PASSWORD', '1234');

//受信するメールの最大容量
define('MAX_SIZE', 1024);

//自動返信メールの送信元アドレス
define('RETURN_ADDRESS', 'example@example.com');

//自動返信メールの送信者名
define('RETURN_NAME', '自動返信');

//自動返信メールの件名
define('RETURN_SUBJECT', '自動返信メール');

//自動返信メールの本文
define('RETURN_BODY', 'これは自動返信メールです。
空メールを送信するとURLが送られてくる…などの仕組みに利用できます。

[$address] からメールを受信しました。');

//ログファイル
define('LOG_FILE', '');

/********* メイン処理 ***********************************************/

$mails = array();
$mail  = 0;
$trash = 0;

//POPサーバーに接続
$sock = fsockopen(POP_SERVER, 110, $errno, $errstr, 30);
if ($sock) {
	$buffer = fgets($sock, 512);
	if (substr($buffer, 0, 3) != '+OK') {
		exit($buffer);
	}

	//ログイン
	$buffer .= sendcmd($sock, 'USER ' . POP_USER);
	$buffer .= sendcmd($sock, 'PASS ' . POP_PASSWORD);

	if (preg_match('/\-ERR/', $buffer)) {
		exit('POPサーバーにログインできません。');
	}

	//メール件数取得
	if (preg_match('/^\+OK\s+(\d+)\s+\d+/', sendcmd($sock, 'STAT'), $matches)) {
		$mail = $matches[1];
	} else {
		$mail = 0;
	}

	for ($i = 1; $i <= $mail; $i++) {
		//メールサイズ取得
		if (preg_match('/^\+OK\s+\d+\s+(\d+)/', sendcmd($sock, 'LIST ' . $i), $matches)) {
			$size = $matches[1];
		} else {
			continue;
		}

		//メールサイズチェック
		if ($size > MAX_SIZE * 1024) {
			$trash++;
		} else {
			//メール取得
			$buffer = sendcmd($sock, 'RETR ' . $i);

			$mails[$i] = '';
			while (!preg_match("/^\.\r\n/", $buffer)) {
				$buffer = fgets($sock, 512);
				$mails[$i] .= $buffer;
			}
		}

		//メール削除
		sendcmd($sock, 'DELE ' . $i);
	}

	//ログアウト
	sendcmd($sock, 'QUIT');

	//接続終了
	fclose($sock);
} else {
	exit('POPサーバーに接続できません。');
}

//メール解析
for ($i = 1; $i <= ($mail - $trash); $i++) {
	//ヘッダーと本文に分割
	list($mail_header, $mail_body) = explode("\r\n\r\n", $mails[$i], 2);

	//本文の不要なホワイトスペースを削除
	$mail_body = preg_replace("/\r\n[\t ]+/", ' ', $mail_body);

	//ヘッダー情報取得
	$headers = get_mailheaders($mail_header);

	//メール送信
	$head  = "X-Mailer: PHP5\n";
	$head .= 'From: "' . mb_encode_mimeheader(mb_convert_kana(RETURN_NAME, 'KV', 'UTF-8')) . '" <' . RETURN_ADDRESS . '>';

	$body = str_replace('[$address]', $headers['address'], RETURN_BODY);

	$subject = mb_convert_kana(RETURN_SUBJECT, 'KV', 'UTF-8');
	$body    = mb_convert_kana(unify($body), 'KV', 'UTF-8');

	$flag = mb_send_mail($headers['address'], $subject, $body, $head);
	if (!$flag) {
		exit('メールの送信に失敗しました。');
	}

	//ログ記録
	if (LOG_FILE) {
		if ($fp = fopen(LOG_FILE, 'a')) {
			fwrite($fp, $headers['date'] . ' に ' . $headers['address'] . ' へメールを返信しました。' . "\n");
			fclose($fp);
		} else {
			exit('ログファイルに記録できません。');
		}
	}
}

exit($mail . '件のメールを処理しました。');

/********* ユーザー定義関数 *****************************************/

/* コマンド送信 */
function sendcmd($sock, $cmd)
{
	fputs($sock, "$cmd\r\n");

	$buffer = fgets($sock, 512);

	return $buffer;
}

/* ヘッダー情報取得 */
function get_mailheaders($header)
{
	$headers = array();

	//メール送信日時取得
	if (preg_match('/\nDate:[ \t]*([^\r\n]+)/', $header, $matches)) {
		$headers['date'] = date('Y-m-d H:i:s', strtotime($matches[1]));
	} else {
		$headers['date'] = date('Y-m-d H:i:s');
	}

	//送信元メールアドレス取得
	if (preg_match('/\nFrom:[ \t]*([^\r\n]+)/', $header, $matches)) {
		$from = $matches[1];
	} elseif (preg_match('/\nReply-To:[ \t]*([^\r\n]+)/', $header, $matches)) {
		$from = $matches[1];
	} elseif (preg_match('/\nReturn-Path:[ \t]*([^\r\n]+)/', $header, $matches)) {
		$from = $matches[1];
	} else {
		$from = null;
	}

	$from = str_replace('"', '', $from);

	if (preg_match('/([\w\.\+\-]+@[\w\.\+\-]+)/', $from, $matches)) {
		$headers['address'] = $matches[1];
	} else {
		$headers['address'] = null;
	}

	return $headers;
}

/* 改行コード統一 */
function unify($data)
{
	if (is_array($data)) {
		return array_map('unify', $data);
	}

	$data = preg_replace("/\r?\n/", "\r", $data);
	$data = preg_replace("/\r/", "\n", $data);

	return $data;
}

?>
